/* tr_main - main pgm to test tree pkg
 */
#include "tree.h"
#define DATA_NODE struct data_node
DATA_NODE
	{
	DATA_NODE *right;
	DATA_NODE *left;
	char data[4];
	};

/* mknode - allocate a new node with specified key
 */
DATA_NODE *mknode(key)
	char key[];
	{
	DATA_NODE *p;

	p = (DATA_NODE *)malloc(sizeof(DATA_NODE));
	if (p == NULL)
		return (NULL);
	strncpy(p->data, key, sizeof(p->data));
	TR_RIGHT(p) = TR_LEFT(p) = NULL;
	return (p);
	}

/*
	tr_pr -- print the contents of the tree
*/
void tr_pr(pt)
	DATA_NODE *pt;	/* ptr to root of tree */
	{
	static short level = -1;
	short i;

	if (pt == NULL)
		return;
	++level;
	tr_pr(TR_LEFT(pt));
	for (i = 0; i < level; ++i)
		putchar('.');
	printf("%s\n", pt->data);
	tr_pr(TR_RIGHT(pt));
	--level;
	}

/*
	comp_str -- comparison function 
*/

int comp_str(p1, p2)
	DATA_NODE *p1;
	DATA_NODE *p2;
	{
	return(strcmp(p1->data, p2->data));
	}


main()
	{

	DATA_NODE *root = NULL;
	DATA_NODE *p = NULL;
	DATA_NODE **q = NULL;
	static char *keys[] = 
		{"d", "b", "c", "a", "e"};
	int i = 0;
	int comp_str();		/* comparison function */

	for (i = 0; i < DIM(keys); ++i)
		{
		p = (DATA_NODE *) mknode(keys[i]);
		TR_INSERT(&root, p, comp_str);
		q = (DATA_NODE **) TR_LFIND(&root, p, comp_str);
		tr_pr(root);
		printf("q=<%s>\n", (*q)->data);
		q = (DATA_NODE **) TR_LPFIND(&root, p, comp_str);
		printf("parent=<%s>\n", *q != NULL ? (*q)->data : "NULL");
		q = (DATA_NODE **) TR_LFIND(&root, p, comp_str);
		TR_DELETE(&root, q, comp_str);
		printf("after deletion\n");
		tr_pr(root);
		printf("after re-inserting\n");
		p = (DATA_NODE *) mknode(keys[i]);
		TR_INSERT(&root, p, comp_str);
		q = (DATA_NODE **) TR_LFIND(&root, p, comp_str);
		tr_pr(root);
		printf("q=<%s>\n\n", (*q)->data);
		}
	p = mknode("a");	/* dup, already in tree */
	TR_INSERT(&root, p, comp_str);
	printf("after dup:\n");
	tr_pr(root);

	p = mknode("x");	/* not in tree */
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	printf("LFIND(missing) %s properly NULL\n", *q == NULL ? "is" : "NOT");

	q = (DATA_NODE **)TR_LPFIND(&root, p, comp_str);
	printf("LPFIND(missing) %s properly NULL\n", *q == NULL ? "is" : "NOT");

	p = mknode("d");	/* the root */
	q = (DATA_NODE **)TR_LPFIND(&root, p, comp_str);
	printf("LPFIND(root) %s properly NULL\n", *q == NULL ? "is" : "NOT");

	p = mknode("d");	/* root, 2 subtrees */
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	q = (DATA_NODE **)TR_LNEXT(&root, *q, comp_str);
	printf("LNEXT(d) == <%s>\n", *q != NULL ? (*q)->data : "NULL");

	p = mknode("a");	/* no subtrees, successor is parent */
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	q = (DATA_NODE **)TR_LNEXT(&root, *q, comp_str);
	printf("LNEXT(a) == <%s>\n", *q != NULL ? (*q)->data : "NULL");

	p = mknode("e");	/* no subtrees, has no successor */
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	q = (DATA_NODE **)TR_LNEXT(&root, *q, comp_str);
	printf("LNEXT(e) == <%s>\n", *q == NULL ? "NULL" : (*q)->data);

	p = mknode("e");	/* no subtrees, has no successor */
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	q = (DATA_NODE **)TR_LFIRST(q);
	printf("LFIRST(e) == <%s>\n", *q == NULL ? "NULL" : (*q)->data);

	p = mknode("d");
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	TR_DELETE(&root, q, comp_str);
	printf("\nAfter deleting 'd':\n");
	tr_pr(root);

	p = mknode("b");
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	TR_DELETE(&root, q, comp_str);
	printf("\nAfter deleting 'b':\n");
	tr_pr(root);

	p = mknode("c");
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	TR_DELETE(&root, q, comp_str);
	printf("\nAfter deleting 'c':\n");
	tr_pr(root);

	p = mknode("a");
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	TR_DELETE(&root, q, comp_str);
	printf("\nAfter deleting 'a':\n");
	tr_pr(root);

	p = mknode("e");
	q = (DATA_NODE **)TR_LFIND(&root, p, comp_str);
	TR_DELETE(&root, q, comp_str);
	printf("\nAfter deleting 'e':\n");
	tr_pr(root);
	}

